#include "stm32f10x.h"
#include "stdio.h"	
#include "rtc.h" 		    
#include "stm32f10x_rtc.h"
#include "stm32f10x_bkp.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_pwr.h"
#include "delay.h" 

//#define RTC_CLOCK_SOURCE_LSE

_calendar_obj calendar;

void RTC_IRQHandler(void)
{		 
	if (RTC_GetITStatus(RTC_IT_SEC) != RESET)
	{							
		//RTC_Get();   
 	}
	if(RTC_GetITStatus(RTC_IT_ALR)!= RESET)
	{
		RTC_ClearITPendingBit(RTC_IT_ALR);			  	   
  	} 				  								 
	RTC_ClearITPendingBit(RTC_IT_SEC|RTC_IT_OW);		
	RTC_WaitForLastTask();	  	    						 	   	 
}

void RTC_Start(void)
{
#ifdef RTC_CLOCK_SOURCE_LSE
	u8 temp=0;
#endif
	//RTC_NVIC_Config();

	if (BKP_ReadBackupRegister(BKP_DR1) != 0x5050)		
	{
		printf("First Init\n");
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);
		PWR_BackupAccessCmd(ENABLE);
		BKP_DeInit();
#ifdef RTC_CLOCK_SOURCE_LSE
		RCC_LSEConfig(RCC_LSE_ON);
		printf("wait RCC_FLAG_LSERDY\n");
		while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET)
		{
			if(++temp>=250)	return;
			delay_ms(10);
		}
		RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);	
		RCC_RTCCLKCmd(ENABLE);
		printf("RTC Wait For LSE Synchro\n");
		RTC_WaitForSynchro();
		RTC_WaitForLastTask();
		RTC_ITConfig(RTC_IT_SEC, ENABLE);
		RTC_WaitForLastTask();
		RTC_EnterConfigMode();
		RTC_SetPrescaler(32767);
#else
		RCC_RTCCLKConfig(RCC_RTCCLKSource_HSE_Div128);	
		RCC_RTCCLKCmd(ENABLE);
		printf("RTC Wait For LSI Synchro\n");
		RTC_WaitForSynchro();
		RTC_WaitForLastTask();
		RTC_ITConfig(RTC_IT_SEC, ENABLE);
		RTC_WaitForLastTask();
		RTC_EnterConfigMode();
		RTC_SetPrescaler(62500);
#endif
		RTC_WaitForLastTask();	
		RTC_Set(2015,8,9,16,58,0);
		RTC_ExitConfigMode(); 
		BKP_WriteBackupRegister(BKP_DR1, 0X5050);
	}
	else
	{
#ifndef RTC_CLOCK_SOURCE_LSE
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);
		PWR_BackupAccessCmd(ENABLE);
		//BKP_DeInit();		
		RCC_RTCCLKConfig(RCC_RTCCLKSource_HSE_Div128);	
		RCC_RTCCLKCmd(ENABLE);	
		printf("RTC Wait For Synchro\n");
		RTC_WaitForSynchro();
		RTC_WaitForLastTask();
		RTC_ITConfig(RTC_IT_SEC, ENABLE);
		RTC_WaitForLastTask();
		RTC_EnterConfigMode();
		RTC_SetPrescaler(62500);		
#endif
		printf("RTC Synchro\n");
		RTC_WaitForSynchro();
		RTC_ITConfig(RTC_IT_SEC, ENABLE);
		RTC_WaitForLastTask();	
	}		    				     
	RTC_Get();
	RCC_ClearFlag();
}

const u8 table_week[12]={0,3,3,6,1,4,6,2,5,0,3,5}; 	  
const u8 mon_table[12]={31,28,31,30,31,30,31,31,30,31,30,31};

u8 Is_Leap_Year(u16 year)
{			  
	if(year%4==0) 
	{ 
		if(year%100==0) 
		{ 
			if(year%400==0)return 1; 	   
			else return 0;   
		}else return 1;   
	}else return 0;	
}

u8 RTC_Get_Week(u16 year,u8 month,u8 day)
{	
	u16 temp2;
	u8 yearH,yearL;
	
	yearH=year/100;	yearL=year%100; 
	 
	if (yearH>19)yearL+=100;
	
	temp2=yearL+yearL/4;
	temp2=temp2%7; 
	temp2=temp2+day+table_week[month-1];
	if (yearL%4==0&&month<3)temp2--;
	return(temp2%7);
}	

u8 RTC_Set(u16 syear,u8 smon,u8 sday,u8 hour,u8 min,u8 sec)
{
	u16 t;
	u32 seccount=0;
	if(syear<1970||syear>2099)return 1;	   
	for(t=1970;t<syear;t++)	
	{
		if(Is_Leap_Year(t))seccount+=31622400;
		else seccount+=31536000;			  
	}
	smon-=1;
	for(t=0;t<smon;t++)	   
	{
		seccount+=(u32)mon_table[t]*86400;
		if(Is_Leap_Year(syear)&&t==1)seccount+=86400;	   
	}
	seccount+=(u32)(sday-1)*86400; 
	seccount+=(u32)hour*3600;
    seccount+=(u32)min*60;	 
	seccount+=sec;

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);	  
	PWR_BackupAccessCmd(ENABLE);	 
	RTC_SetCounter(seccount);	

	RTC_WaitForLastTask();	  	
	return 0;	    
}

u8 RTC_Get(void)
{
	static u16 daycnt=0;
	u32 timecount=0; 
	u32 temp=0;
	u16 temp1=0;	  
 	timecount=RTC->CNTH;
	timecount<<=16;
	timecount+=RTC->CNTL;			 

 	temp=timecount/86400;   
	if(daycnt!=temp)
	{	  
		daycnt=temp;
		temp1=1970;	
		while(temp>=365)
		{				 
			if(Is_Leap_Year(temp1))
			{
				if(temp>=366)temp-=366;
				else {temp1++;break;}  
			}
			else temp-=365;	  
			temp1++;  
		}   
		calendar.w_year=temp1;
		temp1=0;
		while(temp>=28)
		{
			if(Is_Leap_Year(calendar.w_year)&&temp1==1)
			{
				if(temp>=29)temp-=29;
				else break; 
			}
			else 
			{
				if(temp>=mon_table[temp1])temp-=mon_table[temp1];
				else break;
			}
			temp1++;  
		}
		calendar.w_month=temp1+1;	
		calendar.w_date=temp+1;  	 
	}
	temp=timecount%86400;     		   	   
	calendar.hour=temp/3600;     	
	calendar.min=(temp%3600)/60; 		
	calendar.sec=(temp%3600)%60; 	
	calendar.week=RTC_Get_Week(calendar.w_year,calendar.w_month,calendar.w_date);   
	return 0;
}

void RTC_GetTime(uint32_t RTC_Format, RTC_TimeTypeDef* RTC_TimeStruct)
{
	vu16 w_year;
	vu8  w_month;
	vu8  w_date;
	static u16 daycnt=0;
	u32 timecount=0; 
	u32 temp=0;
	u16 temp1=0;	  
 	timecount=RTC->CNTH;
	timecount<<=16;
	timecount+=RTC->CNTL;			 

 	temp=timecount/86400;   
	if(daycnt!=temp)
	{	  
		daycnt=temp;
		temp1=1970;	
		while(temp>=365)
		{				 
			if(Is_Leap_Year(temp1))
			{
				if(temp>=366)temp-=366;
				else {temp1++;break;}  
			}
			else temp-=365;	  
			temp1++;  
		}   
		w_year=temp1;
		temp1=0;
		while(temp>=28)
		{
			if(Is_Leap_Year(w_year)&&temp1==1)
			{
				if(temp>=29)temp-=29;
				else break; 
			}
			else 
			{
				if(temp>=mon_table[temp1])temp-=mon_table[temp1];
				else break;
			}
			temp1++;  
		}
		w_month=temp1+1;	
		w_date=temp+1;  	 
	}
	temp=timecount%86400;     		   	   
	RTC_TimeStruct->RTC_Hours=temp/3600;     	
	RTC_TimeStruct->RTC_Minutes=(temp%3600)/60; 		
	RTC_TimeStruct->RTC_Seconds=(temp%3600)%60; 	
	//calendar.week=RTC_Get_Week(calendar.w_year,calendar.w_month,calendar.w_date);
}
